#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <gl/glut.h>

#include <math.h>
#include <stdio.h>
#include "ParametricCurve.h"

#define PI 3.141593

////////////////////////////
// class CParametricCurve //
////////////////////////////

// Draw n line segments approximating the curve
void CParametricCurve::draw()
{
	glBegin(GL_LINE_STRIP);
	for(int i=0;i<=n;i++)
		glVertex2fv(vertices[i]);
	glEnd();
}

// compute (n+1) points on the curve
void CParametricCurve::init()
{
	for(int i=0;i<=n;i++)
		computePointOnCurve((float) i/(float) n,vertices[i][0],vertices[i][1]);
}

/////////////////////////////
// class CParametricCircle //
/////////////////////////////

CParametricCircle::CParametricCircle()
{
	cx=0.0f; cy=0.0f; r=1.0f;	// unit circle as default parameter
	CParametricCurve::init();
}

CParametricCircle::CParametricCircle(float centreX, float centreY, float radius):cx(centreX),cy(centreY),r(radius)
{
	CParametricCurve::init();
}

void CParametricCircle::computePointOnCurve(float t, float& x, float& y){
	x=cx+r*cos(t*2.0*PI);
	y=cy+r*sin(t*2.0*PI);
}


////////////////////////////////
// class CParametricEllipsoid //
////////////////////////////////

CParametricEllipsoid::CParametricEllipsoid()
{
	cx=0.0f; cy=0.0f; a=1.0f; b=1.0f;	// unit circle as default parameter
	CParametricCurve::init();
}

CParametricEllipsoid::CParametricEllipsoid(float centreX, float centreY, float axis1, float axis2):cx(centreX),cy(centreY),a(axis1),b(axis2)
{
	CParametricCurve::init();
}

void CParametricEllipsoid::computePointOnCurve(float t, float& x, float& y){
	x=cx+a*cos(t*2.0*PI);
	y=cy+b*sin(t*2.0*PI);
}

//////////////////////////////
// class CLogarithmicSpiral //
//////////////////////////////

CLogarithmicSpiral::CLogarithmicSpiral()
{
	cx=150.0f; cy=150.0f; K=10, a=0.1, n=4;	// default parameters to draw something pretty
	CParametricCurve::init();
}

CLogarithmicSpiral::CLogarithmicSpiral(float centreX, float centreY, float myK, float my_a, float nRevolutions):cx(centreX),cy(centreY),K(myK),a(my_a),n(nRevolutions)
{
	CParametricCurve::init();
}

void CLogarithmicSpiral::computePointOnCurve(float t, float& x, float& y){
	double f=K*exp(n*a*t*2.0*PI);
	x=cx+f*cos(n*t*2.0*PI);
	y=cy+f*sin(n*t*2.0*PI);
}

//////////////////////
// class CRoseCurve //
//////////////////////

CRoseCurve::CRoseCurve()
{
	cx=150.0f; cy=150.0f; r=80; n=6;	// default parameters to draw something pretty
	CParametricCurve::init();
}

CRoseCurve::CRoseCurve(float centreX, float centreY, float radius, float nPadels):cx(centreX),cy(centreY),r(radius),n(nPadels)
{
	CParametricCurve::init();
}

void CRoseCurve::computePointOnCurve(float t, float& x, float& y){
	double f=r*cos(n*t*2.0*PI);
	x=cx+f*cos(t*2.0*PI);
	y=cy+f*sin(t*2.0*PI);
}

