#if !defined(LECTURE372_PARAMETRIC_CURVE_DEMO)
#define LECTURE372_PARAMETRIC_CURVE_DEMO

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

const int n=500;		// number of segments for drawing curve

// A virtual class for a 2D parametric curve
// No instances of this class can be constructed
class CParametricCurve
{
public:
	CParametricCurve(){}
	virtual ~CParametricCurve(){}
	void draw();			// draws curve as a line strip
	virtual void computePointOnCurve(float t, float& x, float& y)=0;	
							// computes curve point p(t)=(x(t), y(t))
protected:
	float vertices[n+1][2];
	void init();			// computes n vertices on the curve
};

class CParametricCircle:public CParametricCurve
{
public:
	CParametricCircle();
	CParametricCircle(float centreX, float centreY, float radius);
	virtual ~CParametricCircle(){}
	void computePointOnCurve(float t, float& x, float& y);
private:
	float cx,cy;	// x-coordinate and y-coordinate of the centre
	float r;		// radius	
};

class CParametricEllipsoid:public CParametricCurve
{
public:
	CParametricEllipsoid();
	CParametricEllipsoid(float centreX, float centreY, float axis1, float axis2);
	virtual ~CParametricEllipsoid(){}
	void computePointOnCurve(float t, float& x, float& y);
private:
	float cx,cy;	// x-coordinate and y-coordinate of the centre
	float a,b;		// short and long axis	
};

class CLogarithmicSpiral:public CParametricCurve
{
public:
	CLogarithmicSpiral();
	CLogarithmicSpiral(float centreX, float centreY, float myK, float my_a, float nRevolutions);
	virtual ~CLogarithmicSpiral(){}
	void computePointOnCurve(float t, float& x, float& y);
private:
	float cx,cy;	// x-coordinate and y-coordinate of the centre
	float K,a;
	float n;		// number of revolutions
};

class CRoseCurve:public CParametricCurve
{
public:
	CRoseCurve();
	CRoseCurve(float centreX, float centreY, float radius, float nPadels);
	virtual ~CRoseCurve(){}
	void computePointOnCurve(float t, float& x, float& y);
private:
	float cx,cy;	// x-coordinate and y-coordinate of the centre
	float r;		// radius of the shape
	float n;		// 2n is the number of padels of the shape
};

#endif // !defined(LECTURE372_PARAMETRIC_CURVE_DEMO)
